library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.pscaler_local_package.all;
-- simple frame buffer for nearest neighbour filter
-- streaming input in, and supports single adress readout
-- assumes enough waiting cycles between frames

entity pscaler_buffer is
  
  generic (
    G_PIXEL_DEPTH   : integer := 8;
    G_HMAXOUT_WIDTH : integer := 7;
    G_VMAXOUT_WIDTH : integer := 7);

  port (
    clk         : in  std_logic;
    arst        : in  std_logic;
    buffer_full : out std_logic;

    --write
    datavalid_in : in std_logic;
    data_in      : in std_logic_vector(G_PIXEL_DEPTH-1 downto 0);

    --read
    adress_x      : in  std_logic_vector(G_HMAXOUT_WIDTH-1 downto 0);
    adress_y      : in  std_logic_vector(G_VMAXOUT_WIDTH-1 downto 0);
    reload_buffer : in  std_logic;
    data_out      : out std_logic_vector(G_PIXEL_DEPTH-1 downto 0)
    );
end pscaler_buffer;

architecture arch of pscaler_buffer is
  signal rdaddress        : std_logic_vector(G_HMAXOUT_WIDTH+G_VMAXOUT_WIDTH-1 downto 0);
  signal wraddr_to_buffer : unsigned(G_HMAXOUT_WIDTH+G_VMAXOUT_WIDTH-1 downto 0);
  signal wren             : std_logic;
  signal buffer_full_int  : std_logic;

begin  -- arch

  -----------------------------------------------------------------------------
  -- 16x16 pixel buffer:
  -----------------------------------------------------------------------------
--  pscaler_ram : pscaler_ram
--    port map (
--      clock     => clk,
--      data      => data_in,
--      rdaddress => rdaddress,
--      wraddress => std_logic_vector(wraddr_to_buffer),
--      wren      => wren,
--      q         => data_out);


  -----------------------------------------------------------------------------
  -- 128x128 pixel buffer:
  -----------------------------------------------------------------------------
--  pscaler_ram : pscaler_ram128
--    port map (
--      clock     => clk,
--      data      => data_in,
--      rdaddress => rdaddress,
--      wraddress => std_logic_vector(wraddr_to_buffer),
--      wren      => wren,
--      q         => data_out);

  -----------------------------------------------------------------------------
  -- 128x128 pixel buffer:
  -----------------------------------------------------------------------------
  pscaler_ram : pscaler_ram128
    port map (
      clock     => clk,
      data      => data_in,
      rdaddress => rdaddress(13 downto 0),
      wraddress => std_logic_vector(wraddr_to_buffer(13 downto 0)),
      wren      => wren,
      q         => data_out);

  -----------------------------------------------------------------------------
  -- 256x256 pixel buffer:
  -----------------------------------------------------------------------------
--  pscaler_ram : pscaler_ram256
--    port map (
--      clock     => clk,
--      data      => data_in,
--      rdaddress => rdaddress,
--      wraddress => std_logic_vector(wraddr_to_buffer),
--      wren      => wren,
--      q         => data_out);

  rdaddress   <= adress_y & adress_x;
  buffer_full <= buffer_full_int;
  wren        <= datavalid_in and not buffer_full_int;

  sync : process (clk, arst)
  begin  -- process sync
    if arst = '1' then                  -- asynchronous reset (active high)
      buffer_full_int  <= '0';
      wraddr_to_buffer <= (others => '0');
    elsif clk'event and clk = '1' then  -- rising clock edge
      if reload_buffer = '1' then
        buffer_full_int  <= '0';
        wraddr_to_buffer <= (others => '0');
      else
        if datavalid_in = '1' and buffer_full_int = '0' then
          wraddr_to_buffer <= wraddr_to_buffer+1;
          if wraddr_to_buffer = (wraddr_to_buffer'range => '1') then
            buffer_full_int <= '1';
          end if;
        end if;
      end if;
    end if;
  end process sync;

end arch;
