library ieee;
use ieee.std_logic_1164.all;

package pscaler_local_package is

  constant C_ZERO_IND                : integer := 11;  -- zero precision index. Integer valued vectors should stop here
  constant C_PERSPECTIVE_FACTOR_HIGH : integer := 7;
  constant C_MAX_PRECISION           : integer := 0;
  constant C_MAPPING_PRECISION       : integer := 6;

  component pscaler_top is

    generic (
      G_HMAXOUT_WIDTH : integer := 4;  --* Max number of horizontal output pixels
      --* is (2 ^ G_HMAXOUT_WIDTH)
      G_VMAXOUT_WIDTH : integer := 4;
      G_PIXEL_DEPTH   : integer := 8
      );

    port (
      clk       : in  std_logic;
      arst      : in  std_logic;
      ready     : in  std_logic;
      ready_out : out std_logic;

      image_start  : in std_logic;
      data_in      : in std_logic_vector(7 downto 0);
      datavalid_in : in std_logic;

      -- Perspective scaling attributes
      -- Px, Py, h_start (=0.5), v_start (=0.5), (x_inc_start)
      input_hsize  : in std_logic_vector(G_HMAXOUT_WIDTH-1 downto 0);
      Px           : in std_logic_vector(7 downto 0);

      input_vsize    : in std_logic_vector(G_VMAXOUT_WIDTH-1 downto 0);
      Py             : in std_logic_vector(7 downto 0);
      Py_slope_start : in std_logic_vector(7 downto 0);  -- range 0 -> 2.00

      data_out      : out std_logic_vector(G_PIXEL_DEPTH-1 downto 0);
      datavalid_out : out std_logic);

  end component;

  component pscaler_ram is
    port
      (
        clock     : in  std_logic := '1';
        data      : in  std_logic_vector (7 downto 0);
        rdaddress : in  std_logic_vector (7 downto 0);
        wraddress : in  std_logic_vector (7 downto 0);
        wren      : in  std_logic := '0';
        q         : out std_logic_vector (7 downto 0)
        );
  end component;

  component pscaler_ram128 is
    port
      (
        clock     : in  std_logic := '1';
        data      : in  std_logic_vector (7 downto 0);
        rdaddress : in  std_logic_vector (13 downto 0);
        wraddress : in  std_logic_vector (13 downto 0);
        wren      : in  std_logic := '0';
        q         : out std_logic_vector (7 downto 0)
        );
  end component;

  component pscaler_ram256 is
    port
      (
        clock     : in  std_logic := '1';
        data      : in  std_logic_vector (7 downto 0);
        rdaddress : in  std_logic_vector (15 downto 0);
        wraddress : in  std_logic_vector (15 downto 0);
        wren      : in  std_logic := '0';
        q         : out std_logic_vector (7 downto 0)
        );
  end component;


  component pscaler_buffer is
    generic (
      G_PIXEL_DEPTH   : integer := 8;
      G_HMAXOUT_WIDTH : integer := 7;
      G_VMAXOUT_WIDTH : integer := 7);

    port (
      clk         : in  std_logic;
      arst        : in  std_logic;
      buffer_full : out std_logic;

      --write
      datavalid_in : in std_logic;
      data_in      : in std_logic_vector(G_PIXEL_DEPTH-1 downto 0);

      --read
      adress_x      : in  std_logic_vector(G_HMAXOUT_WIDTH-1 downto 0);
      adress_y      : in  std_logic_vector(G_VMAXOUT_WIDTH-1 downto 0);
      reload_buffer : in  std_logic;
      data_out      : out std_logic_vector(G_PIXEL_DEPTH-1 downto 0)
      );

  end component;

  component pscaler_filter is

    generic (
      G_PIXEL_DEPTH   : integer := 8;  -- Number of bits representing one pixel value
      G_HMAXOUT_WIDTH : integer := 11;  -- Number of bits in the pixel index variable. Max number of pixels is 2^G_INDEX_WIDTH.
      G_VMAXOUT_WIDTH : integer := 11);  -- number of bits representing the decimal value of pixel position
    port (
      clk       : in  std_logic;
      arst      : in  std_logic;
      ready     : in  std_logic;
      ready_out : out std_logic;

      -- frame buffer interface
      frame_buffer_xdat         : out std_logic_vector(G_HMAXOUT_WIDTH+C_ZERO_IND-1 downto C_ZERO_IND);
      frame_buffer_ydat         : out std_logic_vector(G_VMAXOUT_WIDTH+C_ZERO_IND-1 downto C_ZERO_IND);
      frame_buffer_request_data : out std_logic;
      frame_buffer_value        : in  std_logic_vector(G_PIXEL_DEPTH-1 downto 0);

      xPos           : in  std_logic_vector(G_HMAXOUT_WIDTH+C_ZERO_IND-1 downto C_MAPPING_PRECISION);
      yPos           : in  std_logic_vector(G_VMAXOUT_WIDTH+C_ZERO_IND-1 downto C_MAPPING_PRECISION);
      position_valid : in  std_logic;
      outofbounds    : in  std_logic;
      filter_value   : out std_logic_vector(G_PIXEL_DEPTH-1 downto 0);
      datavalid_out  : out std_logic);

  end component;

  component pscaler_mapper is

    generic (
      G_HMAXOUT_WIDTH : integer := 11;
      G_VMAXOUT_WIDTH : integer := 11);

    port (
      clk       : in  std_logic;
      arst      : in  std_logic;
      ready     : in  std_logic;
      ready_out : out std_logic;

      image_start : in std_logic;

      -- Perspective scaling attributes
      -- Px, Py, h_start (=0.5), v_start (=0.5), (x_inc_start)
      input_hsize  : in std_logic_vector(G_HMAXOUT_WIDTH-1 downto 0);
      Px           : in std_logic_vector(C_PERSPECTIVE_FACTOR_HIGH downto C_MAX_PRECISION);

      input_vsize    : in std_logic_vector(G_VMAXOUT_WIDTH-1 downto 0);
      Py             : in std_logic_vector(C_PERSPECTIVE_FACTOR_HIGH downto C_MAX_PRECISION);  --*2^-11
      Py_slope_start : in std_logic_vector(C_ZERO_IND+1 downto C_ZERO_IND-6);  -- = Py*(height-1)/2.

      -- input location (decimal precision) 
      xPos            : out std_logic_vector(G_HMAXOUT_WIDTH+C_ZERO_IND-1 downto C_MAPPING_PRECISION);
      yPos            : out std_logic_vector(G_VMAXOUT_WIDTH+C_ZERO_IND-1 downto C_MAPPING_PRECISION);
      outofbounds     : out std_logic;
      -- output location (integer precision) 
      uPos            : out std_logic_vector(G_HMAXOUT_WIDTH-1 downto 0);
      vPos            : out std_logic_vector(G_VMAXOUT_WIDTH-1 downto 0);
      datavalid_out   : out std_logic;
      mapper_finished : out std_logic
      );

  end component;

end package pscaler_local_package;
