library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.pscaler_local_package.all;

entity pscaler_mapper is

  generic (
    G_HMAXOUT_WIDTH : integer := 11;
    G_VMAXOUT_WIDTH : integer := 11);

  port (
    clk       : in  std_logic;
    arst      : in  std_logic;
    ready     : in  std_logic;
    ready_out : out std_logic;

    image_start : in std_logic;

    -- Perspective scaling attributes
    -- Px, Py, h_start (=0.5), v_start (=0.5), (x_inc_start)
    input_hsize  : in std_logic_vector(G_HMAXOUT_WIDTH-1 downto 0);
    Px           : in std_logic_vector(C_PERSPECTIVE_FACTOR_HIGH downto C_MAX_PRECISION);

    input_vsize    : in std_logic_vector(G_VMAXOUT_WIDTH-1 downto 0);
    Py             : in std_logic_vector(C_PERSPECTIVE_FACTOR_HIGH downto C_MAX_PRECISION);  --*2^-11
    Py_slope_start : in std_logic_vector(C_ZERO_IND+1 downto C_ZERO_IND-6);  -- = Py*(height-1)/2.

    -- input location (decimal precision) 
    xPos            : out std_logic_vector(G_HMAXOUT_WIDTH+C_ZERO_IND-1 downto C_MAPPING_PRECISION);
    yPos            : out std_logic_vector(G_VMAXOUT_WIDTH+C_ZERO_IND-1 downto C_MAPPING_PRECISION);
    outofbounds     : out std_logic;
    -- output location (integer precision) 
    uPos            : out std_logic_vector(G_HMAXOUT_WIDTH-1 downto 0);
    vPos            : out std_logic_vector(G_VMAXOUT_WIDTH-1 downto 0);
    datavalid_out   : out std_logic;
    mapper_finished : out std_logic
    );

end pscaler_mapper;


architecture rtl of pscaler_mapper is

  signal y_slope : signed(C_ZERO_IND+1 downto C_MAX_PRECISION);  --*2^-11 (owerlaps with Py and slope_start)
  signal x_inc   : unsigned(C_ZERO_IND+1 downto C_MAX_PRECISION);  --*2^-11

  -- internal floating point precision position variables, bridging the gap
  -- between x_inc/y_slope and xPos/yPos.
  signal xPos_int : unsigned(G_HMAXOUT_WIDTH+C_ZERO_IND+1 downto C_MAX_PRECISION);  --2 overflow-guard bits
  signal yPos_int : unsigned(G_VMAXOUT_WIDTH+C_ZERO_IND+1 downto C_MAX_PRECISION);  --2 overflow-guard bits
  signal overflow : std_logic;

  signal uPos_int : unsigned(G_HMAXOUT_WIDTH-1 downto 0);
  signal vPos_int : unsigned(G_VMAXOUT_WIDTH-1 downto 0);

  signal y_start             : unsigned(yPos'range);
  signal mapper_started      : std_logic;
  signal mapper_finished_int : std_logic;

  constant one                 : unsigned(30 downto 0)   := to_unsigned(2**(C_ZERO_IND), 31);
  constant zero_point_five     : unsigned(30 downto 0)   := to_unsigned(2**(C_ZERO_IND-1), 31);
  constant y_start_second_iter : unsigned(y_start'range) := to_unsigned(2**(C_ZERO_IND-C_MAPPING_PRECISION-1)+2**(C_ZERO_IND-C_MAPPING_PRECISION), y_start'length);  --=1.5
  constant x_inc_start         : unsigned(x_inc'range)   := to_unsigned(2**(C_ZERO_IND), x_inc'length);  --=1
  
begin

  -- overflow has happened if one of these are high:
  overflow <= xPos_int(xPos_int'high) or xPos_int(xPos_int'high-1) or
              yPos_int(yPos_int'high) or yPos_int(yPos_int'high-1);
  mapper_finished <= mapper_finished_int;

  p_sync : process (clk, arst)
  begin  -- process p_sync
    if arst = '1' then
      mapper_started      <= '0';
      mapper_finished_int <= '0';

      y_start                       <= y_start_second_iter(y_start'range);
      y_slope                       <= (others => '0');
      y_slope(Py_slope_start'range) <= signed(Py_slope_start);
      x_inc                         <= x_inc_start;
      xPos_int                      <= zero_point_five(xpos_int'range);

      yPos_int <= zero_point_five(yPos_int'range);
      uPos_int <= (others => '0');
      vPos_int <= (others => '0');
    elsif rising_edge(clk) then
      if (ready = '1') then
        if image_start = '1' then
          mapper_started                <= '1';
          --reset
          y_start                       <= y_start_second_iter(y_start'range);
          y_slope                       <= (others => '0');
          y_slope(Py_slope_start'range) <= signed(Py_slope_start);
          x_inc                         <= x_inc_start;
          xPos_int                      <= zero_point_five(xpos_int'range);
          yPos_int                      <= zero_point_five(yPos_int'range);
          uPos_int                      <= (others => '0');
          vPos_int                      <= (others => '0');
          
        else

          -----------------------------------------------------------------------
          -- per pixel iteration:
          -----------------------------------------------------------------------
          uPos_int <= uPos_int+1;
          x_inc    <= x_inc + unsigned(Px);
          if overflow = '0' then
            -- if overflow has occured, no further calculations are carried out
            xPos_int <= xPos_int + x_inc;
            yPos_int <= unsigned(signed(yPos_int) - y_slope);
          end if;

          if std_logic_vector(uPos_int) = input_hsize then
            -- per line resets
            uPos_int                <= (others => '0');
            x_inc                   <= x_inc_start;
            xPos_int                <= zero_point_five(xpos_int'range);
            yPos_int                <= (others => '0');
            yPos_int(y_start'range) <= y_start;

            -----------------------------------------------------------------------
            -- per line iteration:
            -----------------------------------------------------------------------
            vPos_int <= vPos_int + 1;
            y_slope  <= y_slope - signed('0' & Py);
            y_start  <= y_start + one(y_start'range);
            if std_logic_vector(vPos_int) = input_vsize then
              mapper_finished_int <= '1';
            end if;
          end if;
        end if;
      end if;
      -- stops the mapper after one interation (until new "start_image")
      if mapper_finished_int = '1' then
        mapper_started      <= '0';
        mapper_finished_int <= '0';
      end if;
    end if;

  end process p_sync;

  outofbounds   <= overflow;
  datavalid_out <= mapper_started;
  xPos          <= std_logic_vector(xPos_int(xPos'range));
  yPos          <= std_logic_vector(yPos_int(yPos'range));
  uPos          <= std_logic_vector(uPos_int);
  vPos          <= std_logic_vector(vPos_int);


end rtl;
