library ieee;
use ieee.std_logic_1164.all;

--library PerspectiveScaler;
use work.pscaler_local_package.all;

entity pscaler_top is
  generic (
    G_HMAXOUT_WIDTH : integer := 8;  --* Max number of horizontal output pixels
    --* is (2 ^ G_HMAXOUT_WIDTH)
    G_VMAXOUT_WIDTH : integer := 8;
    G_PIXEL_DEPTH   : integer := 8
    );

  port (
    clk       : in  std_logic;
    arst      : in  std_logic;
    ready     : in  std_logic;
    ready_out : out std_logic;

    image_start  : in std_logic;
    data_in      : in std_logic_vector(7 downto 0);
    datavalid_in : in std_logic;

    -- Perspective scaling attributes
    -- Px, Py, h_start (=0.5), v_start (=0.5), (x_inc_start)
    input_hsize  : in std_logic_vector(G_HMAXOUT_WIDTH-1 downto 0);
    Px           : in std_logic_vector(7 downto 0);

    input_vsize    : in std_logic_vector(G_VMAXOUT_WIDTH-1 downto 0);
    Py             : in std_logic_vector(7 downto 0);
    Py_slope_start : in std_logic_vector(7 downto 0);  -- range 0 -> 2.00

    data_out      : out std_logic_vector(G_PIXEL_DEPTH-1 downto 0);
    datavalid_out : out std_logic);

end pscaler_top;

architecture rtl of pscaler_top is

  signal xPos : std_logic_vector(G_HMAXOUT_WIDTH+C_ZERO_IND-1 downto C_MAPPING_PRECISION);
  signal yPos : std_logic_vector(G_VMAXOUT_WIDTH+C_ZERO_IND-1 downto C_MAPPING_PRECISION);

  signal uPos : std_logic_vector(G_HMAXOUT_WIDTH-1 downto 0);
  signal vPos : std_logic_vector(G_VMAXOUT_WIDTH-1 downto 0);

  signal value_from_filter     : std_logic_vector(G_PIXEL_DEPTH-1 downto 0);
  signal datavalid_from_filter : std_logic;
  signal datavalid_from_mapper : std_logic;

  signal buffer_full    : std_logic;
  signal start_mapper   : std_logic;
  signal mapper_started : std_logic;

  signal x_addr_from_filter      : std_logic_vector(G_HMAXOUT_WIDTH-1 downto 0);
  signal y_addr_from_filter      : std_logic_vector(G_VMAXOUT_WIDTH-1 downto 0);
  signal data_from_buffer        : std_logic_vector(G_PIXEL_DEPTH-1 downto 0);
  signal outofbounds_from_mapper : std_logic;
  signal mapper_finished         : std_logic;
  signal reload_buffer : std_logic;
  
begin  -- rtl

  p_fsm : process (buffer_full)
  begin  -- process p_fsm
    if buffer_full = '1' then
      ready_out <= '0';
    else
      ready_out <= '1';
    end if;
  end process p_fsm;

  p_sync : process (clk, arst)
  begin  -- process p_sync
    if arst = '1' then
      mapper_started <= '0';
    elsif clk'event and clk = '1' then
      -- default values:
      start_mapper  <= '0';
      reload_buffer <= '0';

      if buffer_full = '1' and mapper_started = '0' then
        start_mapper   <= '1';
        mapper_started <= '1';
      end if;
      if mapper_finished = '1' then
        reload_buffer  <= '1';
        mapper_started <= '0';
      end if;
    end if;
  end process p_sync;

  pscaler_buffer_1 : pscaler_buffer
    generic map (
      G_PIXEL_DEPTH   => G_PIXEL_DEPTH,
      G_HMAXOUT_WIDTH => G_HMAXOUT_WIDTH,
      G_VMAXOUT_WIDTH => G_VMAXOUT_WIDTH)
    port map (
      clk          => clk,
      arst         => arst,
      buffer_full  => buffer_full,
      datavalid_in => datavalid_in,
      data_in      => data_in,
      adress_x     => x_addr_from_filter,
      adress_y     => y_addr_from_filter,
      data_out     => data_from_buffer,
		reload_buffer => reload_buffer);

  pscaler_mapper_1 : pscaler_mapper
    generic map (
      G_HMAXOUT_WIDTH => G_HMAXOUT_WIDTH,
      G_VMAXOUT_WIDTH => G_VMAXOUT_WIDTH)
    port map (
      clk             => clk,
      arst            => arst,
      ready           => ready,
      ready_out       => open,
      image_start     => start_mapper,
      input_hsize     => input_hsize,
      Px              => Px,
      input_vsize     => input_vsize,
      Py              => Py,
      Py_slope_start  => Py_slope_start,
      xPos            => xPos,
      yPos            => yPos,
      uPos            => uPos,
      vPos            => vPos,
      outofbounds     => outofbounds_from_mapper,
      datavalid_out   => datavalid_from_mapper,
      mapper_finished => mapper_finished);


  pscaler_filter_1 : pscaler_filter
    generic map (
      G_PIXEL_DEPTH   => G_PIXEL_DEPTH,
      G_HMAXOUT_WIDTH => G_HMAXOUT_WIDTH,
      G_VMAXOUT_WIDTH => G_VMAXOUT_WIDTH)
    port map (
      clk                       => clk,
      arst                      => arst,
      ready                     => ready,
      ready_out                 => open,
      frame_buffer_xdat         => x_addr_from_filter,
      frame_buffer_ydat         => y_addr_from_filter,
      frame_buffer_request_data => open,
      frame_buffer_value        => data_from_buffer,
      xPos                      => xPos,
      yPos                      => yPos,
      position_valid            => datavalid_from_mapper,
      outofbounds               => outofbounds_from_mapper,
      filter_value              => data_out,
      datavalid_out             => datavalid_out);

end rtl;
